#!/usr/bin/env Rscript

### Bash requirements (via module load or specified in OOD start up box)
# module load gcc/8.1.0
# module load udunits/2.2.26_gcc8.1.0
# module load geos/3.7.1
# module unload R
# module load 
# module load R/4.2.2-openblas
# module load hdf5
# cd /home/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/


### Do some checks that the setup is correct:
# Should return 4.2.2 and some extra info
version 

system("which R")

### Make sure it is looking in the correct location for libraries
.libPaths()

### Load required packages
### If needed, install packages to my library, since the shared one at /panfs/roc/msisoft/R/4.2.2-openblas/lib64/R/library is not writable.
# install.packages('Seurat', lib='/panfs/jay/groups/3/lmnp/bmcc/R/x86_64-pc-linux-gnu-library/4.2')

# BiocManager::install("biomaRt", lib='/panfs/jay/groups/3/lmnp/bmcc/R/x86_64-pc-linux-gnu-library/4.2')


library(Seurat) #cran
library(glue) 
library(dplyr)
library(purrr)
library(readr)
library(stringr)
library(msigdbr)


#######################################################################
# Script parameters
#######################################################################
# Clear out any objects from workspace
rm(list = ls(all.names = TRUE))

# Set R working directory and project name here.
### !!! Make sure proj_dir and project_name are set for the specific capture
proj_dir <- "/home/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502"
setwd(proj_dir)

project_name <- "C1C2_NoCreVlyveCre"

# Set up output directory and filename prefix
out_dir <- glue("{proj_dir}/code_out/043_merge_and_filter")
if (!dir.exists(glue("{out_dir}"))) {dir.create(glue("{out_dir}"), recursive = TRUE)}
setwd(glue("{out_dir}"))
prefix <- "043_"
if (!dir.exists(out_dir)) {dir.create(out_dir, recursive = TRUE)}


#######################################################################
# Analysis
#######################################################################


# ---------------------------------------------------------------------
# Read input data
# ---------------------------------------------------------------------

### Since we have two separate toaster runs, I have to combine the CellRanger output stored in filtered_feature_bc_matrix/
### I am processing them separately, then using merge() later.

C1_start <- read_rds(file = "/home/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/code_out/031_filter_by_rna/031_seurat_object.rds")

C2_start <- read_rds(file = "/home/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/code_out/032_filter_by_rna_C2/032_C2_seurat_object.rds")

curr_seurat_object <- merge(x=C1_start, y = C2_start, add.cell.ids = c("C1", "C2"), project = project_name)

rm(C1_start); rm(C2_start)

NoCreCells <- curr_seurat_object$HTO_maxID %in% c("tum1NoCre", "tum2NoCre")
curr_seurat_object$genotype <- "LyveCre"
curr_seurat_object$genotype[NoCreCells] <- "NoCre"
rm(NoCreCells)

capture1Cells <- curr_seurat_object$HTO_maxID %in% c("tum1NoCre", "tum1LyveCre")
curr_seurat_object$capture <- "C2"
curr_seurat_object$capture[capture1Cells] <- "C1"
rm(capture1Cells)

Idents(curr_seurat_object) <- "HTO_classification"
RidgePlot(curr_seurat_object, assay = "HTO", features = rownames(curr_seurat_object[["HTO"]]), ncol = 2)

#saveRDS(curr_seurat_object, file = glue("{out_dir}/{prefix}curr_seurat_object.rds"))


### For GEO data submission:
write.table(t(as.matrix(curr_seurat_object@assays$RNA@counts)), file =  glue("{out_dir}/Raw_RNA_Counts.tsv"))
write.table(t(as.matrix(curr_seurat_object@assays$HTO@counts)), file =  glue("{out_dir}/Raw_HTO_Counts.tsv"))
### In the next part, we use this to get the filtered, SCTransformed counts:
# write.table(t(as.matrix(curr_seurat_object@assays$SCT@counts)), file =  glue("{out_dir}/Normalized_RNA_Counts.tsv"))



# ---------------------------------------------------------------------
# QC Plots and filtering
# ---------------------------------------------------------------------

DefaultAssay(curr_seurat_object) <- "RNA"
Idents(curr_seurat_object) <- "HTO_classification.global"

table(curr_seurat_object@meta.data$HTO_classification)
barplot(table(curr_seurat_object@meta.data$HTO_classification))

pdf(glue("{out_dir}/{prefix}qc_metrics_violin.pdf"), width = 10, height = 6)
VlnPlot(curr_seurat_object, features = c("nFeature_RNA", "nCount_RNA", "percent_mito"), ncol = 3, pt.size = 0)
dev.off()


m <- curr_seurat_object@meta.data$percent_mito
m1 <- quantile(as.numeric(m), c(0.99))
pdf(glue("{out_dir}/{prefix}qc_percent_mito.pdf"), width = 10, height = 6)
plot(m, main = "percent_mito, 99.5% thresholds", ylab = "percent_mito", xlab = "cell_id")
abline(h = m1[1])
abline(h = m1[2])
dev.off()



plot1 <- FeatureScatter(curr_seurat_object, feature1 = "nCount_RNA", feature2 = "percent_mito")
plot2 <- FeatureScatter(curr_seurat_object, feature1 = "nCount_RNA", feature2 = "nFeature_RNA")
pdf(glue("{out_dir}/{prefix}qc_metrics_scatter.pdf"), width = 8, height = 6)
# FYI: Plots are combined via "patchwork" R package methods
plot1 + plot2
dev.off()


f <- curr_seurat_object@meta.data$nFeature_RNA
f1 <- quantile(as.numeric(f), c(0.01, 0.99))
pdf(glue("{out_dir}/{prefix}qc_nFeature_RNA.pdf"), width = 8, height = 6)
plot(f, main = "nFeature_RNA, 0.5% and 99.5% thresholds", ylab = "nFeature_RNA", xlab = "cell_id")
abline(h = f1[1])
abline(h = f1[2])
dev.off()


c <- curr_seurat_object@meta.data$nCount_RNA
c1 <- quantile(as.numeric(c), c(0.01, 0.99))
pdf(glue("{out_dir}/{prefix}qc_nCount_RNA.pdf"), width = 8, height = 6)
plot(c, main = "nCount_RNA, 2% and 98% thresholds", ylab = "nCount_RNA", xlab = "cell_id")
abline(h = c1[1])
abline(h = c1[2])
dev.off()






















### !!! ___ ___ ___ BOOKMARK ___ ___ ___ !!!
# THIS IS WHERE WE NEED TO SUBSET THE seurat_object to only have the singlets with OK percent mt, etc.


# ---------------------------------------------------------------------
# Subset the data
# ---------------------------------------------------------------------

# Identify cells that pass mito cutoff
#OK_percent_mito <- curr_seurat_object@meta.data$percent_mito <= m1
OK_percent_mito <- curr_seurat_object@meta.data$percent_mito <= 5

# Identify cells that are singlets
OK_singlets <- curr_seurat_object@meta.data$HTO_classification.global == "Singlet"

# Keep only cells that pass the mito threshold
#curr_seurat_object <- curr_seurat_object[, OK_percent_mito]
curr_seurat_object <- curr_seurat_object[, OK_singlets & OK_percent_mito]


# ---------------------------------------------------------------------
# Check object sizes
# ---------------------------------------------------------------------

dim(curr_seurat_object@assays$RNA@counts)
dim(curr_seurat_object@assays$HTO@counts)

#dim(curr_seurat_object1@assays$RNA@counts)
#dim(curr_seurat_object1@assays$HTO@counts)



# THIS IS WHERE WE NEED TO SUBSET THE seurat_object to only have the singlets with OK percent mt, etc.







### !!! ___ ___ ___ BOOKMARK ___ ___ ___ !!!
# Finalize filtering







# ---------------------------------------------------------------------
# Run standard log normalization on the RNA assay
# ---------------------------------------------------------------------
#curr_seurat_object <- NormalizeData(curr_seurat_object, assay = "RNA", normalization.method = "LogNormalize", verbose = TRUE)
#curr_seurat_object <- FindVariableFeatures(curr_seurat_object, assay = "RNA", selection.method = "vst", nfeatures = 3000, verbose = TRUE)
#curr_seurat_object <- ScaleData(curr_seurat_object, assay = "RNA", features = rownames(curr_seurat_object), vars.to.regress = c("nCount_RNA", "percent_mito"))
# ---------------------------------------------------------------------
# Run SCT transformation (first time)
# ---------------------------------------------------------------------
# How to use SCT with cell cycle regression
# https://github.com/satijalab/seurat/issues/1679#issuecomment-557781838
# Run sctransform
# Results are saved in a new assay (named SCT by default) with counts being (corrected)
# counts, data being log1p(counts), scale.data being pearson
# residuals; sctransform::vst intermediate results are saved in misc
# slot of new assay.
# Will probably get warnings: "In theta.ml(y = y, mu = fit$fitted) : iteration limit reached" 
# This is not a problem:
# https://github.com/satijalab/seurat/issues/1378
# UNRELIABLE VALUE: Future (future_lapply-2) unexpectedly generated random numbers without specifying argument future.seed...
# This is not a problem:
# https://github.com/satijalab/seurat/issues/3622

### For testing:
#curr_seurat_object <- SCTransform(curr_seurat_object, assay = "RNA", verbose = FALSE, return.only.var.genes = FALSE)
#write_rds(curr_seurat_object, file = glue("{out_dir}/{prefix}temp_RNA_seurat_object.rds"))

### To get SCT values:
date()
curr_seurat_object <- SCTransform(curr_seurat_object, assay = "RNA", vars.to.regress = "percent_mito", verbose = FALSE, return.only.var.genes = FALSE)
date()


#saveRDS(curr_seurat_object, file = glue("{out_dir}/{prefix}curr_seurat_object.rds"))


#######################################################################
# Save session info
#######################################################################

# ---------------------------------------------------------------------
# Save the Seurat Object
# ---------------------------------------------------------------------
write_rds(curr_seurat_object, file = glue("{out_dir}/{prefix}seurat_object.rds"))

# ---------------------------------------------------------------------
# Save all R objects
# ---------------------------------------------------------------------

# save.image(file = glue("{out_dir}/{prefix}r_objects.RData"))
write_tsv(data.frame(RData_objects = ls()), glue("{out_dir}/{prefix}r_objects.txt"))
