## ---- setup, include=FALSE----------------------------------------------------
knitr::opts_knit$set(root.dir = '/panfs/jay/groups/20/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/')


## ---- Braedan Notes, include=FALSE--------------------------------------------
### Some of this is based on modifying Todd's workflows here: https://github.umn.edu/knut0297org/farrarm_todd_citeseq_nilotinib_pdl1_20210109/blob/663ae278f1b2aa0b0b36c762f61a31fec193e272/code/

### Clear all the objects in memory
rm(list = ls(all.names = TRUE))

### Set this to TRUE to run all of the time-intensive steps or FALSE if you have done so previously and want to skip them. Saves lots of time when knitting since it just loads prior .rds files.
RUNALLSTEPS <- TRUE

### !!! Specify Round of clustering, etc. here !!! ___ Allow for specifying via command line or sourced document in future versions.
Round <- "Round0" ; res_to_use <- 0.6 ; clusters_to_use <- c(0:1,3:6,8:16)
#Round <- "Round1" ; res_to_use <- 0.6 ; clusters_to_use <- c(1,3,7,9)
#Round <- "Round2" ; res_to_use <- 0.6 ; clusters_to_use <- c(0:10)



### This script should be able to run several times with minor tweaks in the "Set Parameters" block. It will do a number of things:
### 1) Load a seurat object and user-defined parameters for e.g. gene lists, cluster resolutions, minimum percent of cells expressing a gene
### 2) Perform cluster-independent analyses (findNeighbors(), plot UMAPS, and featurePlot() according to sample)
### 3) Cluster across resolutions defined in "all_resolutions" 
### 4) Perform cluster-dependent analyses, which will require choosing a clustering resolution ()
### 5)

### Notes on Round of clustering:
# Round0 will check if there are any sample-specific clusters that could represent contamination. If none are identified, nothing will happen and we will proceed to Round 1. 
#   Key output: bargraphs with proportion of cells per cluster across sample. 
#   Key decisions: Which resolution gives the best partitioning of sample-specific clusters? Which clusters to remove?

# Round1 is the standard clustering for an scRNA-seq experiment. Clusters identified in Round0 as contamination are removed, then clusters are built using all remaining cells from all treatments.
#   Key output: UMAPs, 
#   Key decisions: 

# Round2 is the final round of clustering only the cells from the population of interest (e.g. all macrophages) to find subtypes within that population. 
#   Key output: 
#   Key decisions: 




## ---- Load packages, include=FALSE--------------------------------------------
# ---------------------------------------------------------------------
# Load R packages
# ---------------------------------------------------------------------
print(Sys.time()); print("")
print("Loading packages"); 

set.seed(20)

options(
  rlang_backtrace_on_error = "full",
  error = rlang::entrace,
  menu.graphics = FALSE
)

library(tidyverse)
library(glue)
library(Seurat) 
library(gridExtra)
library(cowplot)
library(msigdbr)
library(ggforce)
library(clustree)
library(viridis)
#library(sctransform)
library(limma)
library(clusterProfiler)




## ---- Set Parameters----------------------------------------------------------

#######################################################################
# Script parameters
#######################################################################
print(Sys.time()); print(glue("\n"))
print(glue("Checking user-supplied parameters")); 

# Set R working directory
proj_dir <- "/home/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502"

### This is the resolution passed to FindClusters(). 0.5 seems to work well. In this stage, we just want something big enough to separate any weird clusters (comtamination, doublets, etc.) 
all_resolutions <- c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1)

### These parameters are for FindMarkers() and FindAllMarkers().
min.pct <- 0.1
logfc.threshold <- 0.1

print(glue("Round = {Round}"))
print(glue("res_to_use = {res_to_use}"))
print(glue("clusters_to_use = {glue_collapse(clusters_to_use, sep = ',')} ")); 

if(!exists("Round")){
  stop("Must specify clustering round using e.g. 'Round <- Round1'")
}

if(!Round %in% c("Round0","Round1","Round2")){
  stop('Round must be specified as one of: "Round0","Round1","Round2"')
}

if(Round == "Round0"){
# Set up output directory and filename prefix
out_dir <- glue("{proj_dir}/code_out/071_Round0_cluster/")
if (!dir.exists(glue("{out_dir}"))) {dir.create(glue("{out_dir}"), recursive = TRUE)}
#setwd(glue("{out_dir}"))
prefix <- "071_"
}

if(Round == "Round1"){
# Set up output directory and filename prefix
out_dir <- glue("{proj_dir}/code_out/081_Round1_cluster/")
if (!dir.exists(glue("{out_dir}"))) {dir.create(glue("{out_dir}"), recursive = TRUE)}
#setwd(glue("{out_dir}"))
prefix <- "081_"
}

if(Round == "Round2"){
# Set up output directory and filename prefix
out_dir <- glue("{proj_dir}/code_out/091_Round2_cluster/")
if (!dir.exists(glue("{out_dir}"))) {dir.create(glue("{out_dir}"), recursive = TRUE)}
#setwd(glue("{out_dir}"))
prefix <- "091_"
}


# Set color scheme
blueX2_grayX2 <- c("tum1LyveCre" = "deepskyblue2", "tum1NoCre" = "gray50", "tum2LyveCre" = "royalblue3", "tum2NoCre" = "gray30")
featurePlotColors <- c("lightgrey", "blue")



## -----------------------------------------------------------------------------
#######################################################################
# Analysis
#######################################################################
print(Sys.time()); print("");
print("Reading input data"); 

# ---------------------------------------------------------------------
# Read input data
# ---------------------------------------------------------------------

### !!! May tweak later if doing different cell cycle normalization or splitting cells before this step.

# Get sample names info
#gem_groups_nice <- read_tsv(glue("{proj_dir}/code_out/051_subset_by_sample/051_gem_groups_nice.txt")) %>% 
#  pull(1)

### Skipping above because we only have one group
#gem_groups_nice <- "AllCells"

curr_sample_name <- "AllCells"
curr_cell_cycle_norm_method <- c("cc_none")
# cell_cycle_norm_method <- c("cc_none", "cc_s_and_g2m", "cc_s_minus_g2m")

if(Round == "Round0"){
#curr_seurat_object <- read_rds(glue("{proj_dir}/code_out/061_transform/{curr_sample_name}/{curr_cell_cycle_norm_method}/061_seurat_object.rds"))
curr_seurat_object <- read_rds(glue("{proj_dir}/code_out/043_merge_and_filter/043_seurat_object.rds"))

  
# Store the "HTO_maxID" in a NEW COLUMN of meta.data called "Sample" to make things more transparent.
curr_seurat_object$Sample <- curr_seurat_object$HTO_maxID
}

if(Round == "Round1"){
#curr_seurat_object <- read_rds(glue("{proj_dir}/code_out/071_Round0_cluster/{curr_sample_name}/{curr_cell_cycle_norm_method}/071_seurat_object.rds"))
  
  ### !!! For now:

curr_seurat_object <- read_rds(glue("{proj_dir}/CodeOutput/Round0Clusters/SavedSeuratObjects/071_seurat_object.rds"))
  
}

if(Round == "Round2"){
#curr_seurat_object <- read_rds(glue("{proj_dir}/code_out/081_Round1_cluster/{curr_sample_name}/{curr_cell_cycle_norm_method}/081_seurat_object.rds"))
curr_seurat_object <- read_rds(glue("{proj_dir}/CodeOutput/Round1Clusters/SavedSeuratObjects/081_seurat_object.rds"))
  
}



## ---- Specify gene modules, echo=FALSE,warning=FALSE,message=FALSE,error=FALSE, results='hide',fig.keep='all'----
print(Sys.time()); print("")
print("Specifying gene modules"); 

### This is a list of cell type markers and others that have come up for cell typing or specific requests from Alexis. We will NOT use this as a gene list, but WILL make plots for each of these individually.
### "Lyve1" removed because of super low expression that causes errors.
genes_for_FeaturePlot <- c("Il1b", "H2-Ab1", "H2-Eb1", "H2-Aa", "Ccl2", "C1qa", "Ccl7", "Ccl12", "Ccnd1", "Cd3e", "Cd4", "Cd8a", "Foxp3", "Ms4a1", "Nkg7", "Ptprc", "Cd14", "Csf1r", "Cd19", "S100a9", "Hspa8", "Dynll1", "Dnaja1", "Hyal1", "Hyal2", "C1qa", "C1qb", "C1qc", "Folr2", "Mrc1", "Stab1", "Gas6")

#genes_for_FeaturePlot <- c("Ms4a1","Csf1r","C1qa")


# ---------------------------------------------------------------------
# Get cell cycle genes and add metadata with AddModuleScore()
# ---------------------------------------------------------------------

# A list of cell cycle markers, from Tirosh et al, 2015, is loaded with Seurat.  We can
# segregate this list into markers of G2/M phase and markers of S phase
# The cc.genes vector is from the Seurat package
s_phase <- tibble(human_gene_symbol = Seurat::cc.genes$s.genes,
                  marker_class = "s_phase")
g2m_phase <- tibble(human_gene_symbol = Seurat::cc.genes$g2m.genes,
                    marker_class = "g2m_phase")
cell_cycle_human <- bind_rows(s_phase, g2m_phase)

# Use the msigdbr R package because it includes annotated orthologs
human_mouse_homologs <- msigdbr::msigdbr(species = "mouse")

human_mouse_homologs_s_phase <- human_mouse_homologs %>%
  dplyr::filter(human_gene_symbol %in% s_phase$human_gene_symbol) %>%
  distinct(gene_symbol, human_gene_symbol) %>%
  dplyr::rename(mouse_gene_symbol = "gene_symbol")

human_mouse_homologs_g2m_phase <- human_mouse_homologs %>%
  dplyr::filter(human_gene_symbol %in% g2m_phase$human_gene_symbol) %>%
  distinct(gene_symbol, human_gene_symbol) %>%
  dplyr::rename(mouse_gene_symbol = "gene_symbol")

# Remove human_mouse_homologs
### Gene modules to use:

SphaseGenes <- pull(human_mouse_homologs_s_phase, mouse_gene_symbol)

G2MphaseGenes <- pull(human_mouse_homologs_g2m_phase, mouse_gene_symbol)

Ibrahim20Fig1CellTypeGenes <- c("Ptprc","Cd3e", "Cd14", "Ms4a1", "Nkg7")

Ibrahim20Fig2MacroGenes <- c("Adgre1", "Cd68", "Csf1r", "Mafb", "Cd14")

AlexisGenes <- c("Lyve1","Timd4", "Stab1",  "Gas6", "Folr2", "Mrc1", "Ednrb", "Fcna", "C1qtnf1", "Maf")

Wang20Sup2_C4PosMarkerGenes <- c("Cd209f","Fcna","Folr2","Cd209g","Lyve1","Cbr2","Gas6","C1qc","C1qa","C1qb","Ednrb","Pf4","Ccl24","Hal","Ccl8","C4b","Clec3b","Pla2g2d","Fcgrt","Rnase4","Ltc4s","Pmp22","F13a1","Mrc1","Fxyd2","Ninj1","Calml4","Ms4a7","Fam213b","Apoe","Igf1","Cd209b","Cfh","Stab1","Maf","Pltp","Dab2","Timd4","Selenbp1","Serpinb6a","Retnla","Mafb","Tslp","Rcn3","Timp2","Wfdc17","Cd209d","Clec10a","Selenop","Grn","Ctsb","Lgmn","Csf1r","Ccl6","Tcn2","Blvrb","Emp1","Cd63","Abca1","Ifitm2","Mt1","Wwp1","Itm2b","Trf","Camk1","Tns1","Ifitm3","Ifi207","C5ar1","Trem2","Ftl1","Fth1","Cfp","Cd302","Zfyve21","Ctsl","Mgst1","Ccl9","C3ar1","Hexa","Ifi27l2a","Tmem37","Pld4","Fcgr2b","Fcgr3")

Wang20Sup3_Lyve1pos_genes_up_set <- c("Fcna","Cd209f","Cd209g","Lyve1","Folr2","Cbr2","C1qa","Ltc4s","C1qc","C1qb","Fcgrt","Gas6","Pf4","Ednrb","Apoe","Hal","Ccl24","Ccl8","Itm2b","Clec3b","F13a1","C4b","Wfdc17","Mrc1","Pla2g2d","Rnase4","Ctsb","Pmp22","Pltp","Maf","Grn","Serpinb6a","Cfh","Rcn3","Timd4","Calml4","Cd209d","Cd209b","Timp2","Igf1","Dab2","Clec10a","Stab1","Ifi27l2a","Selenbp1","Tcn2","Ly6e","Ctsl","Tns1","C6","Csf1r","Zfyve21","Enpep","Ctsd","Camk1","Mtss1","Pdgfc","Snx2","Igfbp4","Itsn1","Pepd","Glul","Frmd4b","Cd5l","Nrp1","Slco2b1","Cystm1","Npl","Colec12","Sulf2","C1qtnf1","Gpt2","Hpse","Rab11fip5","Plekhg5","Cp","Cndp2","Nxpe5","Idh2","Agtrap","Pros1","Gprc5c","Stard8","Ddx60","Kank2","Tmem8","Hmox1","Ehd4","Hyal2","Etv1","Oasl2","Dennd2a","Fam43a","Nagk","Cd163","Gm4951","Tpp1","Uaca","Cmklr1","Ncoa4","Lrpap1","Bmp2","Serpinb1a","Tgfbr2","Optn","Egfr","Nfxl1","Oas2","Ctnnd1","Prune2")

Wang20Sup3_Lyve1pos_genes_down_set <- c("Nfkbie","Syne1","Foxp4","Rasa2","Atp2a3","Plcg2","L1cam","Adora2a","Plcb2","St8sia6","Dennd4a","Il1b","Cadm1","Dusp5","Map3k14","Psd4","Odc1","Skil","Ccl5","Net1","Rab11fip1","Cd24a","Fam49a","Syngr2","Gpr65","Pik3r5","Dna2","Fam129a","Tmem123","Klrb1a","Svil","Rel","Gpr183","Adam8","Alkbh1","Actn1","Myo1g","Il1rl1","Traf1","Samsn1","Bcl2a1b","Rasal3","Cbfa2t3","Gpr132","Arl5c","Tnfaip3","Fam46c","Tbc1d1","Bcl2a1a","Dock8","Gramd1a","Galnt6","Ahcyl2","Arl4c","Itga6","Kif21b","Rin3","Tnip3","Cd5","Ciita","Rgs1","Ccr7","Alcam","Srgn","Lsp1","H2-DMa","Rpl41","Id2","Cytip","H2-DMb1","Ccnd2","Cd74","H2-Aa","H2-Ab1","Rps19","Rps24","Tmsb4x")

gene_modules <- list(AlexisGenes=AlexisGenes, Ibrahim20Fig2MacroGenes=Ibrahim20Fig2MacroGenes, Wang20Sup2_C4PosMarkerGenes=Wang20Sup2_C4PosMarkerGenes, Wang20Sup3_Lyve1pos_genes_up_set=Wang20Sup3_Lyve1pos_genes_up_set, Wang20Sup3_Lyve1pos_genes_down_set=Wang20Sup3_Lyve1pos_genes_down_set, SphaseGenes=SphaseGenes, G2MphaseGenes=G2MphaseGenes)


### Getting gene lists (MH, M2, M5) needed for GSEA since clusterprofiler does not have them yet.

### !!! This did not work. Had to download and upload with transmit  
if(FALSE){
#download.file(url = "http://www.gsea-msigdb.org/gsea/msigdb/download_file.jsp?filePath=/msigdb/release/2023.1.Mm/mh.all.v2023.1.Mm.symbols.gmt", destfile = "/panfs/jay/groups/20/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/input/mh.all.v2023.1.Mm.symbols.gmt")
  
#download.file(url = "http://www.gsea-msigdb.org/gsea/msigdb/download_file.jsp?filePath=/msigdb/release/2023.1.Mm/m2.all.v2023.1.Mm.symbols.gmt", destfile = "/panfs/jay/groups/20/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/input/m2.all.v2023.1.Mm.symbols.gmt")

#download.file(url = "http://www.gsea-msigdb.org/gsea/msigdb/download_file.jsp?filePath=/msigdb/release/2023.1.Mm/m5.all.v2023.1.Mm.symbols.gmt", destfile = "/panfs/jay/groups/20/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/input/m5.all.v2023.1.Mm.symbols.gmt")

#download.file(url = "http://www.gsea-msigdb.org/gsea/msigdb/download_file.jsp?filePath=/msigdb/release/2023.1.Mm/m8.all.v2023.1.Mm.symbols.gmt", destfile = "/panfs/jay/groups/20/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/input/m8.all.v2023.1.Mm.symbols.gmt")

}
  
Mh <- read.gmt(gmtfile = "/panfs/jay/groups/20/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/input/mh.all.v2023.1.Mm.symbols.gmt")

M2 <- read.gmt(gmtfile = "/panfs/jay/groups/20/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/input/m2.all.v2023.1.Mm.symbols.gmt")

M5 <- read.gmt(gmtfile = "/panfs/jay/groups/20/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/input/m5.all.v2023.1.Mm.symbols.gmt")

M8 <- read.gmt(gmtfile = "/panfs/jay/groups/20/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/input/m8.all.v2023.1.Mm.symbols.gmt")




## -----------------------------------------------------------------------------
print(Sys.time()); 
print("Running SCTransform()"); 


if(RUNALLSTEPS == FALSE){ print("Skipping this step.") } ;
if(RUNALLSTEPS == TRUE){


curr_seurat_object <- SCTransform(curr_seurat_object, assay = "RNA", vars.to.regress = "percent_mito", verbose = FALSE, return.only.var.genes = FALSE)

print("Finished SCTransform()"); 
print(Sys.time()); print("");


# ---------------------------------------------------------------------
# Generate PCA, TSNE, and UMAP 
# ---------------------------------------------------------------------
# All of these methods use the "scale.data" slot of the respective assay for input data
# SCT assay
# These are now standard steps in the Seurat workflow for visualization and clustering
print(Sys.time());
print("Running RunPCA()"); 
curr_seurat_object <- RunPCA(curr_seurat_object, assay = "SCT", verbose = FALSE, reduction.name = "pca")
print(Sys.time()); print("")
print("Finished RunPCA()"); 

  Sys.time()
print("Running RunTSNE()"); 
curr_seurat_object <- RunTSNE(curr_seurat_object, assay = "SCT", reduction = "pca", dims = 1:30, reduction.name = "tsne")
print(Sys.time()); print("")
print("Finished RunTSNE()"); 

print(Sys.time());
print("Running RunUMAP()"); 
curr_seurat_object <- RunUMAP(curr_seurat_object, assay = "SCT", reduction = "pca", dims = 1:30, verbose = FALSE, reduction.name = "umap", umap.method = "umap-learn", metric = "correlation")
print(Sys.time()); print("")
print("Finished RunUMAP()"); 
# might get lots of warnings from the umap-learn, but it seems to work!


# ---------------------------------------------------------------------
# Find Neighbors
# ---------------------------------------------------------------------
DefaultAssay(curr_seurat_object) <- "SCT"
  Sys.time()
print("Running FindNeighbors()"); 
curr_seurat_object <- FindNeighbors(curr_seurat_object, reduction = "pca", dims = 1:30, verbose = FALSE)
print(Sys.time()); print("")
print("Finished FindNeighbors()"); 

write_rds(curr_seurat_object, file = glue("{out_dir}{prefix}curr_seurat_object.rds"))
print(glue(".rds file saved as: ", "{out_dir}{prefix}curr_seurat_object.rds"))

} else {

  curr_seurat_object <- read_rds(glue("{out_dir}{prefix}curr_seurat_object.rds"))

}



## -----------------------------------------------------------------------------
print(Sys.time()); print("")
print("Starting DimPlot() for UMAP"); 

if(RUNALLSTEPS == FALSE){ print("Skipping this step.") };  
if(RUNALLSTEPS == TRUE){

# Plot the plain UMAP colored by Sample
clustersByInitialUMAP <- DimPlot(curr_seurat_object, label=FALSE, group.by = "HTO_maxID", cols = blueX2_grayX2)
clustersByInitialUMAP 
ggsave(filename = glue(out_dir, "{Round}Clusters_UMAPBySample.pdf"), device = "pdf", units = "in", width = 8, height = 6)

# Save the UMAP components as easily accessible metadata features.
curr_seurat_object$UM1 <- curr_seurat_object[["umap"]]@cell.embeddings[,"UMAP_1"]
curr_seurat_object$UM2 <- curr_seurat_object[["umap"]]@cell.embeddings[,"UMAP_2"]

# Split it by Sample
DimPlot(curr_seurat_object, split.by = "HTO_maxID", group.by = "HTO_maxID", ncol = 2, cols = blueX2_grayX2) & NoLegend() & NoAxes()
ggsave(filename = glue(out_dir, "{Round}Clusters_UMAP_SplitByGenotype.pdf"), device = "pdf", units = "in", width = 12, height = 8)

# Plot expression of marker features.

}


## ---- Plot Module Scores, echo=FALSE,warning=FALSE,message=FALSE,error=FALSE, results='hide',fig.keep='all'----
print(Sys.time()); print("")
print(glue("Starting FeaturePlot() for ", length(gene_modules), " gene modules and ", length(genes_for_FeaturePlot), " genes")); 

if(RUNALLSTEPS == FALSE){ print("Skipping this step.") }; 
if(RUNALLSTEPS == TRUE){
  #gene_modules
  #genes_for_FeaturePlot
  
  out_dir <- glue("{getwd()}/CodeOutput/{Round}Clusters/FeaturePlotsTogetherAndSplitByGenotype/")
  if(!dir.exists({out_dir})) {dir.create(glue("{out_dir}"), recursive = TRUE)}
  
  
  for(i in 1:length(gene_modules)){
    curr_seurat_object <- AddModuleScore(curr_seurat_object, name = names(gene_modules[i]), features = gene_modules[i], ctrl = lengths(gene_modules)[i], seed=1)
    colnames(curr_seurat_object@meta.data)[which(colnames(curr_seurat_object@meta.data) == glue(names(gene_modules)[i],"1"))] <- names(gene_modules)[i]
  }
  
  
  #if(exists("curr_seurat_object")){curr_seurat_object <- NULL; rm(curr_seurat_object)}
  #curr_seurat_object  <- read_rds(file = glue(getwd(),"/code_out/101_Round1_cluster_analysis/AllCells/cc_none/curr_seurat_object.rds"))
  
  ### Plot the two genotypes next to each other to see the difference for each module.
  #Idents(curr_seurat_object) <- "Sample"
  Idents(curr_seurat_object) <- "genotype"
  
  ### Plot the gene_modules first. Individual genes will be plotted after this loop.
  ### Using split.by="Sample" will combine plots, but omit the legend bar. I am running it with and without, together and split to give lots of options while keeping all graph outputs the same size, etc. (split graphs will be twice as wide)
  
  for(i in 1:length(gene_modules)){
    print(glue("Start ", names(gene_modules)[i]))
    ### Make the feature plots for all the cells together using "curr_seurat_object"
    print(FeaturePlot(curr_seurat_object, features = names(gene_modules)[i], cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F))
    ### To save plot, repeat the command below.
    pdf(glue(out_dir, Round, "Clusters_FeaturePlot_AllCellsTogether_",names(gene_modules[i]),".pdf"), width = 8, height = 8)
      print(FeaturePlot(curr_seurat_object, features = names(gene_modules)[i], cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F))
    dev.off()
  
    ### Make the same plot, but with split.by="Sample". There will be no legend, but the legend in the previous graphs will be applicable.
    print(FeaturePlot(curr_seurat_object, features = names(gene_modules)[i], cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F, split.by="genotype", ncol=2))
    ### To save plot, repeat the command below.
    pdf(glue(out_dir, Round, "Clusters_FeaturePlot_AllCellsSplitByGenotype_",names(gene_modules[i]),".pdf"), width = 32, height = 8)
      print(FeaturePlot(curr_seurat_object, features = names(gene_modules)[i], cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F, split.by="genotype", ncol=2))
    dev.off()
  }  
  
    ### Now subset just one Cre sample into "plot_seurat_object" and repeat the above lines.  
  for(j in unique(Idents(curr_seurat_object))){
    plot_seurat_object <- subset(curr_seurat_object, idents = j)
    for(i in 1:length(gene_modules)){
     print(glue("Start ", names(gene_modules)[i], " for ",j))
  # Already done in previous chunk
  #    plot_seurat_object <- AddModuleScore(plot_seurat_object, name = names(gene_modules[i]), features = gene_modules[i], ctrl = lengths(gene_modules)[i], seed=1)
      colnames(plot_seurat_object@meta.data)[which(colnames(plot_seurat_object@meta.data) == glue(names(gene_modules)[i],"1"))] <- names(gene_modules)[i]
      print(
        FeaturePlot(plot_seurat_object, features = names(gene_modules)[i], cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F)
        )
    ### To save plots
      pdf(glue(out_dir,Round," Clusters_FeaturePlot_AllCellsTogether_",names(gene_modules[i]),"_",j,".pdf"), width = 8, height = 8)
        print(FeaturePlot(plot_seurat_object, features = names(gene_modules)[i], cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F))
      dev.off()
  
    }
  #print(VlnPlot(curr_seurat_object, features = names(gene_modules)[i], pt.size=0.1, split.by = "Sample", split.plot = TRUE))
  }
  
  
  
  
  
  ### Plot the genes_for_FeaturePlot second.
  ### Using split.by="Sample" will combine plots, but omit the legend bar. I am running it with and without, together and split to give lots of options while keeping all graph outputs the same size, etc. (split graphs will be twice as wide)
  
  for(i in genes_for_FeaturePlot){
    print(glue("Start ", i," for all cells"))
    ### Make the feature plots for all the cells together using "curr_seurat_object"
    print(FeaturePlot(curr_seurat_object, features = i, cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F))
    ### To save plot, repeat the command below.
    pdf(glue(out_dir,Round,"Clusters_FeaturePlot_AllCellsTogether_",i,".pdf"), width = 8, height = 8)
      print(FeaturePlot(curr_seurat_object, features = i, cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F))
    dev.off()
    
    ### Make the same plot, but with split.by="Sample". There will be no legend, but the legend in the previous graphs will be applicable.
      print(FeaturePlot(curr_seurat_object, features = i, cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F, split.by="genotype", ncol=2))
    ### To save plot, repeat the command below.
    pdf(glue(out_dir,Round,"Clusters_FeaturePlot_AllCellsSplitByGenotype_",i,".pdf"), width = 32, height = 8)
      print(FeaturePlot(curr_seurat_object, features = i, cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F, split.by="genotype", ncol=2))
    dev.off()
  }
  
    ### Now subset just one Cre sample into "plot_seurat_object" and repeat the above lines.  
  for(j in unique(Idents(curr_seurat_object))){
    plot_seurat_object <- subset(curr_seurat_object, idents = j)
    for(i in genes_for_FeaturePlot){
      print(glue("Start ", i, " for ",j))
  #    colnames(plot_seurat_object@meta.data)[which(colnames(plot_seurat_object@meta.data) == glue(i,"1"))] <- i
      print(
        FeaturePlot(plot_seurat_object, features = i, cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F)
        )
    ### To save plots
      pdf(glue(out_dir,Round,"Clusters_FeaturePlot_",i,"_",j,".pdf"), width = 8, height = 8)
        print(FeaturePlot(plot_seurat_object, features = i, cols = c("lightgrey", "blue"), order = TRUE, pt.size=1.25, label = F))
      dev.off()
  
    }
  #print(VlnPlot(curr_seurat_object, features = i, pt.size=0.1, split.by = "Sample", split.plot = TRUE))
  }
  
  
  
}



## ---- Cluster across resolutions and plot UMAPs-------------------------------
print(Sys.time()); print("")
print(glue("Starting FindClusters() for ", length(all_resolutions), " resolutions.")); 

if(RUNALLSTEPS == FALSE){ print("Skipping this step.") };  
if(RUNALLSTEPS == TRUE){
  
  out_dir <- glue("{getwd()}/CodeOutput/{Round}Clusters/UMAPs_and_BarPlots/")
  if(!dir.exists({out_dir})) {dir.create(glue("{out_dir}"), recursive = TRUE)}
  
  
  for(i in all_resolutions){
    curr_resolution <- i
    curr_res_name <- glue("SCT_snn_res_{curr_resolution}")
    
    curr_seurat_object <- FindClusters(curr_seurat_object, verbose = FALSE, resolution = curr_resolution)
  
    
  # !!! THIS APPEARS TO BREAK IT:  Idents(curr_seurat_object) <- glue("SCT_snn_res.{curr_resolution}")
  
    DimPlot(curr_seurat_object, label = TRUE) + labs(title = glue("UMAP projection\n{curr_sample_name}\n{curr_cell_cycle_norm_method}\n{curr_res_name}")) 
    ggsave(filename = glue(out_dir, "{prefix}SCT_UMAP_res{curr_resolution}.pdf"), width = 8, height = 8, units = "in")
    
  
   
    curr_seurat_object$Cluster <- Idents(curr_seurat_object)
    Idents(curr_seurat_object) <- curr_seurat_object$Cluster
    
    bars <- as.data.frame(prop.table(table(curr_seurat_object$Cluster, curr_seurat_object$HTO_maxID), margin = 2))
    colnames(bars) <- c("Cluster","Sample","Freq")
    ggplot(bars, aes(fill=Sample, y=Freq, x=Cluster)) + geom_bar(position="dodge", stat="identity") + 
      scale_fill_manual(values = c("tum1LyveCre" = "deepskyblue2", "tum1NoCre" = "gray50", "tum2LyveCre" = "royalblue3", "tum2NoCre" = "gray30"))
    ggsave(filename = glue(out_dir, "{Round}Clusters_ClusterCompBySample_res", curr_resolution, ".pdf"), device = "pdf", units = "in", width = 6, height = 6)
    
    
  }
  
  write_rds(x=curr_seurat_object, file = glue("{getwd()}/code_out/{prefix}{Round}_cluster/{prefix}seurat_object.rds"))
  print(glue(".rds file with clusters across all_resolutions saved as: ", "{getwd()}/code_out/{prefix}{Round}_cluster/{prefix}seurat_object.rds"))

} else {

  curr_seurat_object <- read_rds(file = glue("{getwd()}/code_out/{prefix}{Round}_cluster/{prefix}seurat_object.rds"))

}



## ---- Make clustrees----------------------------------------------------------
print(Sys.time()); print("")
print(glue("Plotting Clustrees"));

if(RUNALLSTEPS == FALSE){ print("Skipping this step.") };  
if(RUNALLSTEPS == TRUE){
  
#if(Round == "Round0" | Round == "Round1"){
  
  out_dir <- glue("{getwd()}/CodeOutput/{Round}Clusters/ClustreePlots/")
  if(!dir.exists({out_dir})) {dir.create(glue("{out_dir}"), recursive = TRUE)}
  
  clustree(curr_seurat_object, prefix = "SCT_snn_res.")
  ggsave(filename = glue(out_dir, "{Round}Clusters_SCT_UMAP_clustree.pdf"), device = "pdf", units = "in", width = 8, height = 8)

  for(i in c(names(gene_modules), genes_for_FeaturePlot)){
  #for(i in c(names(gene_modules))){
    
  #  pdf(glue(out_dir, prefix, "SCT_UMAP_clustree_res", ".pdf"), width = 8, height = 8)
    clustree(curr_seurat_object, prefix = "SCT_snn_res.", node_colour = i, node_colour_aggr = "mean", node_size_range = c(2,10), edge_arrow = FALSE, edge_width = 0.5, node_text_colour = "white")
  #  clustree(curr_seurat_object, prefix = "SCT_snn_res.", node_colour = "Wang20Sup2_C4PosMarkerGenes", node_colour_aggr = "mean", node_size_range = c(2,10), node_label_nudge	= 50, edge_arrow = FALSE, edge_width = 0.5, node_text_colour = "white")
    
  #  dev.off()
    ggsave(filename = glue(out_dir, "{Round}Clusters_SCT_UMAP_clustree_", i, ".pdf"), device = "pdf", units = "in", width = 8, height = 8)
  
  #  pdf(glue("{prefix}SCT_UMAP_clustree_C1qa.pdf"), width = 8, height = 8)
  #  clustree(curr_seurat_object, prefix = "SCT_snn_res.", assay = "SCT", exprs = "scale.data", node_colour = "C1qa", node_colour_aggr = "mean", layout = 'tree', prop_filter = 0.1)
  #  clustree(curr_seurat_object, prefix = "SCT_snn_res.", assay = "SCT", exprs = "scale.data", node_colour = "C1qa", node_colour_aggr = "mean", layout = 'tree', prop_filter = 0.1)
    #node_size = "C1qa", node_size_aggr = "mean", node_size_range = c(0,10), 
    #dev.off()
  }

# }

}


## -----------------------------------------------------------------------------
print(Sys.time()); print("")
print(glue("Saving output for future use here: ", glue("{getwd()}/CodeOutput/{Round}Clusters/SavedSeuratObjects/{prefix}seurat_object.rds")));

if(RUNALLSTEPS == FALSE){ print("Skipping this step.") };  
if(RUNALLSTEPS == TRUE){

#if(!dir.exists(glue("{proj_dir}/code_out/{prefix}Round0_cluster/{curr_sample_name}/{curr_cell_cycle_norm_method}/"))) {dir.create(glue("{proj_dir}/code_out/{prefix}Round0_cluster/{curr_sample_name}/{curr_cell_cycle_norm_method}/"))}

out_dir <- glue("{getwd()}/CodeOutput/{Round}Clusters/SavedSeuratObjects/")
if(!dir.exists({out_dir})) {dir.create(glue("{out_dir}"), recursive = TRUE)}


if(!exists("res_to_use") | !exists("clusters_to_use")){
  stop("Define 'res_to_use` and 'clusters_to_use'")
} 

if(is.null(res_to_use) | is.null(clusters_to_use) ){
  stop("'res_to_use` and 'clusters_to_use' cannot be NULL")
}

#######################################################################
# Find top markers per CLUSTER
#######################################################################

### Set the identities to the correct clustering resolution
### DOES NOT WORK: Idents(curr_seurat_object) <- glue("SCT_snn_res.{res_to_use}")
### So reclustering with "res_to_use" specified at the beginning of the script.
### Whenever FindClusters() is run, the clusters are assigned to "curr_seurat_object$seurat_clusters"
curr_seurat_object <- FindClusters(curr_seurat_object, verbose = FALSE, resolution = res_to_use)


DimPlot(curr_seurat_object, label = T)

    curr_seurat_object$Cluster <- Idents(curr_seurat_object) ;
    Idents(curr_seurat_object) <- "Cluster"

    
    
#Idents(curr_seurat_object) <- "seurat_clusters"


### ___ !!! BOOKMARK This is where the error is. Round1Cluster is being set to curr_seurat_object$SCT_snn_res.1 instead of the clusters specified with "res_to_use"

#curr_seurat_object$Cluster <- Idents(curr_seurat_object)
#Idents(curr_seurat_object) <- "Cluster"

if(Round == "Round0"){ curr_seurat_object$Round0Cluster <- Idents(curr_seurat_object) }
if(Round == "Round1"){ curr_seurat_object$Round1Cluster <- Idents(curr_seurat_object) }
if(Round == "Round2"){ curr_seurat_object$Round2Cluster <- Idents(curr_seurat_object) }

write_rds(x=curr_seurat_object, file = glue("{getwd()}/code_out/{prefix}{Round}_cluster/{prefix}seurat_object.rds"))
print(glue(".rds file with {Round}Cluster at resolution {res_to_use} saved as: ", "{getwd()}/code_out/{prefix}{Round}_cluster/{prefix}seurat_object.rds"))


#######################################################################
# Subset to clusters_to_use for next round of clustering
#######################################################################

#clusters_to_exclude <- levels(curr_seurat_object$Cluster)[(!levels(curr_seurat_object$Cluster) %in% clusters_to_use)]
#use_seurat_object <- levels(curr_seurat_object$Cluster)[(!levels(curr_seurat_object$Cluster) %in% clusters_to_use)]

use_seurat_object     <- subset(curr_seurat_object, Cluster %in% clusters_to_use)

exclude_seurat_object <- subset(curr_seurat_object, Cluster %in% clusters_to_use, invert=TRUE)


write_rds(x=use_seurat_object, file = glue("{out_dir}{prefix}seurat_object.rds"))

write_rds(x=exclude_seurat_object, file = glue("{out_dir}{prefix}excluded_clusters_seurat_object.rds"))

}


## -----------------------------------------------------------------------------
print(Sys.time()); print("")
print(glue("Starting FindAllMarkers() for resolution = {res_to_use}."));
print(glue("Keeping clusters {glue_collapse(clusters_to_use, sep = ',')} for future analysis."));

if(RUNALLSTEPS == FALSE){ print("Skipping this step.") };  
if(RUNALLSTEPS == TRUE){


out_dir <- glue("{getwd()}/CodeOutput/{Round}Clusters/ClusterMarkersAndHeatmaps/")
if(!dir.exists({out_dir})) {dir.create(glue("{out_dir}"), recursive = TRUE)}

### Find all markers
## min.pct and logfc.threshold are defined near the beginning.
ALL.markers <- FindAllMarkers(curr_seurat_object, only.pos = TRUE, min.pct = min.pct, logfc.threshold = logfc.threshold)

### Note: lowercase cluster here because that is what FindAllMarkers outputs
TOP100.markers <- ALL.markers %>% group_by(cluster) %>% slice_max(n = 100, order_by = avg_log2FC)

write.table(x=TOP100.markers, file = glue(out_dir,"{Round}Clusters_top100MarkerGenes_ForHeatmap_230101.tsv"), sep = "\t")

### Check the top 5 genes for each cluster
TOP5.markers <- TOP100.markers %>% group_by(cluster) %>% slice_max(n = 5, order_by = avg_log2FC)
TOP5.markers



#cluster.averages <- AverageExpression(curr_seurat_object, return.seurat = TRUE, group.by = "Cluster")



# remove all SNN metadata (not currently working)
#for(i in all_resolutions){
#  to_remove <- glue("SCT_snn_res.",i)
#  test_seurat_object[[]] <- test_seurat_object[[]] %>% mutate("SCT_snn_res.{j}" := NULL)
#}

} else {
  
curr_seurat_object <-  read_rds(file = glue("{getwd()}/code_out/{prefix}{Round}_cluster/{prefix}seurat_object.rds"))
  
}


## ---- Heatmaps and/or DotPlots------------------------------------------------
print(Sys.time()); print("")
print(glue("Making heatmaps"));

if(RUNALLSTEPS == FALSE){ print("Skipping this step.") };  
if(RUNALLSTEPS == TRUE){

out_dir <- glue("{getwd()}/CodeOutput/{Round}Clusters/ClusterMarkersAndHeatmaps/")
if(!dir.exists({out_dir})) {dir.create(glue("{out_dir}"), recursive = TRUE)}

### We have to use top_n() for the heatmaps instead of using slice_max() like we did to generate e.g. TOP20.markers$gene , etc.

### Load the full dataset
#rm(curr_seurat_object)
#curr_seurat_object <- read_rds("/Users/minnow/Documents/Proj/schwertf/Schwertf_Project_028_AE_scRNAseq_22May/seurat_schwertfProj028/C2/code_out/101_Round1_cluster_analysis/AllCells/cc_none/curr_seurat_object.rds")

#curr_seurat_object <- read_rds(glue("{getwd()}/CodeOutput/{Round}Clusters/SavedSeuratObjects/{prefix}seurat_object.rds"))

### Set the heatmap parameters
min.pct.heatmap = 0.1 #0.25
logfc.threshold.heatmap = 0.5 #0.25
top.n.markers = 10


ALL.markers.heatmap <- FindAllMarkers(curr_seurat_object, only.pos = TRUE, min.pct = min.pct.heatmap, logfc.threshold = logfc.threshold.heatmap)

ALL.markers.heatmap %>%
    group_by(cluster) %>%
    top_n(n = top.n.markers, wt = avg_log2FC) -> heatmap.to.plot

### To save plots
    pdf(glue(out_dir,"{Round}Clusters_Heatmap_min.pct", min.pct.heatmap, "_logfc.threshold", logfc.threshold.heatmap,"_top",top.n.markers, "markers.pdf"), width = 8, height = 8)
      print(DoHeatmap(curr_seurat_object, features = heatmap.to.plot$gene))
    dev.off()

#DoHeatmap(curr_seurat_object, features = heatmap.to.plot$gene) 
# + NoLegend()

#ALL.markers.heatmap %>%
#    group_by(cluster) %>%
#    top_n(n = 1, wt = avg_log2FC) -> dotplot.to.plot

#DotPlot(object = curr_seurat_object, features = unique(dotplot.to.plot$gene), )

}



## ---- 221117 requests---------------------------------------------------------
print(Sys.time()); print("");
print(glue("Making VlnPlot()s for individual genes"));

if(RUNALLSTEPS == FALSE){ print("Skipping this step.") };  
if(RUNALLSTEPS == TRUE){

out_dir <- glue("{getwd()}/CodeOutput/{Round}Clusters/SingleGenes_VlnPlots/")
if(!dir.exists({out_dir})) {dir.create(glue("{out_dir}"), recursive = TRUE)}

### •	Violin plots of clusters 1, 2, and 9 from Round 1 clustering split by LyveCre/NoCre for a subset of genes.
#if(exists("curr_seurat_object")){curr_seurat_object <- NULL; rm(curr_seurat_object)}
#curr_seurat_object  <- read_rds(file = "/Users/minnow/Documents/Proj/schwertf/Schwertf_Project_028_AE_scRNAseq_22May/seurat_schwertfProj028/C2/code_out/101_Round1_cluster_analysis/AllCells/cc_none/curr_seurat_object.rds")


if(Round == "Round0"){ Idents(curr_seurat_object) <- "Round0Cluster" }
if(Round == "Round1"){ Idents(curr_seurat_object) <- "Round1Cluster" }
if(Round == "Round2"){ Idents(curr_seurat_object) <- "Round2Cluster" }

levels(Idents(curr_seurat_object))

### o	Genes to include: IL1b, H2-Ab1, H2-Eb1, H2-Aa, Ccl2, C1qa, ccl7, ccl12
genes_to_include <-  genes_for_FeaturePlot

for(i in 1:length(genes_to_include)){
  print(
    VlnPlot(curr_seurat_object, features = genes_to_include[i], pt.size=0.0, split.by = "genotype", split.plot = TRUE, cols=blueX2_grayX2)
    )
  ### To save plots
  pdf(glue(out_dir, "{Round}Clusters_VlnPlot_",genes_to_include[i],".pdf"), width = 8, height = 8)
    print(VlnPlot(curr_seurat_object, features = genes_to_include[i], pt.size=0.0, split.by = "genotype", split.plot = TRUE, cols=blueX2_grayX2))
  dev.off()
}

}



## ---- Plotting Module Scores--------------------------------------------------
print(Sys.time()); print("");
print(glue("Making VlnPlot()s for gene modules"));

if(RUNALLSTEPS == FALSE){ print("Skipping this step.") };  
if(RUNALLSTEPS == TRUE){

out_dir <- glue("{getwd()}/CodeOutput/{Round}Clusters/GeneModules_VlnPlots/")
if(!dir.exists({out_dir})) {dir.create(glue("{out_dir}"), recursive = TRUE)}

if(Round == "Round0"){ Idents(curr_seurat_object) <- "Round0Cluster" }
if(Round == "Round1"){ Idents(curr_seurat_object) <- "Round1Cluster" }
if(Round == "Round2"){ Idents(curr_seurat_object) <- "Round2Cluster" }

levels(Idents(curr_seurat_object))
gene_modules <- list(AlexisGenes=AlexisGenes, Ibrahim20Fig2MacroGenes=Ibrahim20Fig2MacroGenes, Wang20Sup2_C4PosMarkerGenes=Wang20Sup2_C4PosMarkerGenes, Wang20Sup3_Lyve1pos_genes_up_set=Wang20Sup3_Lyve1pos_genes_up_set, Wang20Sup3_Lyve1pos_genes_down_set=Wang20Sup3_Lyve1pos_genes_down_set, SphaseGenes=SphaseGenes, G2MphaseGenes=G2MphaseGenes)

ctrl <- min(vapply(X = gene_modules, FUN = length, FUN.VALUE = numeric(length = 1)))

for(i in 1:length(gene_modules)){
  curr_seurat_object <- AddModuleScore(curr_seurat_object, name = names(gene_modules[i]), features = gene_modules[i], ctrl = lengths(gene_modules)[i], seed=1)
  colnames(curr_seurat_object@meta.data)[which(colnames(curr_seurat_object@meta.data) == glue(names(gene_modules)[i],"1"))] <- names(gene_modules)[i]

### FeaturePlots are now done in a different chunk below.    
#  print(FeaturePlot(curr_seurat_object, features = names(gene_modules)[i],  order = TRUE, pt.size=1.25, split.by = "genotype")  & NoLegend() & NoAxes())
#  ggsave(filename = glue(out_dir, "{Round}Clusters_FeaturePlot_",names(gene_modules[i]),".pdf"), device = "pdf", units = "in", width = 6, height = 6)
  
  print(VlnPlot(curr_seurat_object, features = names(gene_modules)[i], pt.size=0.0, split.by = "genotype", split.plot = TRUE, cols=blueX2_grayX2))
  ggsave(filename = glue(out_dir, "{Round}Clusters_VlnPlot_",names(gene_modules[i]),".pdf"), device = "pdf", units = "in", width = 6, height = 6)

}

#FeaturePlot(curr_seurat_object, features = "percent_mito",  order = TRUE, pt.size=0.25)  & NoLegend() & NoAxes() & theme(plot.title = element_text(size=8))
#FeaturePlot(curr_seurat_object, features = "percent_mito",  order = TRUE, pt.size=1.25, split.by = "genotype")  & NoLegend() & NoAxes()
#FeaturePlot(curr_seurat_object, features = "percent_mito",  order = TRUE, pt.size=0.25, label = T)

  print(VlnPlot(curr_seurat_object, features = "percent_mito", pt.size=0.0, split.by = "genotype", split.plot = TRUE, cols=blueX2_grayX2))
  ggsave(filename = glue(out_dir, "{Round}Clusters_VlnPlot_","percent_mito",".pdf"), device = "pdf", units = "in", width = 6, height = 6)

}


## ---- Set up variables for DGE and GSEA using the Round1 Clusters-------------
print(Sys.time()); print("");
print(glue("Starting INTERcluster GSEA"));

out_dir <- glue("{getwd()}/CodeOutput/{Round}Clusters/GSEA_input/")
if(!dir.exists({out_dir})) {dir.create(glue("{out_dir}"), recursive = TRUE)}

if(RUNALLSTEPS == FALSE) { curr_seurat_object <- read_rds(file = glue("{getwd()}/code_out/{prefix}{Round}_cluster/{prefix}seurat_object.rds")) }



if(RUNALLSTEPS == TRUE){ print("Using curr_seurat_object from previous block")}

### We are using the Round1 clusters, but will do a similar thing for the Round2 clusters in the next file.

### We are going to use the same approach for doing comparisons between clusters (using only cells with genotype = "NoCre") and between samples (using only cells with Cluster = "#", repeated for each cluster).
### This will require subsetting by genotype, then applying identities for Cluster or subsetting by Cluster, then applying identities for genotype

### !!! Import the Sample values into "ident.1" (for positive fold change) and "ident.2" (for negative fold change)
### Generally, "ident.1" will be the "treatment" or "experimental" group.
print(unique(curr_seurat_object$Sample))
#ident.1=c("tum1LyveCre", "tum2LyveCre")
#ident.2=c("tum1NoCre", "tum2NoCre")

ident.1="LyveCre"
ident.2="NoCre"

#print(levels(curr_seurat_object$Round1_Cluster))

### Min percent of cells expressing a gene for it to be compared.
min.pct <- 0.01
### This threshold is the min value reported and is log2(), e.g. 0.25 will return all log2fc's with absolute values >= 0.25
### !!! This was previously coded as being passed to a log2() function. The value here should already be log2(). Reasonable values would be 0.25 for small, but numerous changes or 2 for only very large changes. 
logfc.threshold <- 0.0

#dir.create("GSEA_input", showWarnings = FALSE)
#dir.create("GSEA_output", showWarnings = FALSE)


## ---- Find Markers for INTERcluster GSEA using only NoCre cells---------------
print(Sys.time()); print("");
print(glue("Starting FindAllMarkers() to make .rnk files for INTERcluster GSEA"));
print(glue("TIME INTENSIVE"));

### !!! Add outdir for this block


if(RUNALLSTEPS == FALSE){ print("Skipping this step.") };  
if(RUNALLSTEPS == TRUE){

### BMM edit 220830: Now using signed_negLog10_p_val instead of signed_negLog10_p_val_adj since the latter gave so many 0s and thus no way to order many genes

### We will do a similar task twice (should make a function for doing in the future): once for ALL Round1 Clusters vs each other and once for each of the Macrophage Clusters vs the other Macrophage clusters. I will subset to just NoCre cells, run the first round of analyses, then subset to only the macrophage clusters and run the second round of analyses.

#SubClusters <- c("1","2","5","9")
### Redoing this using "clusters_to_use" specified at the top of this markdown.
clusters_to_use

### Load the full dataset
#rm(curr_seurat_object)
#curr_seurat_object <- read_rds(glue(getwd(),"/code_out/101_Round1_cluster_analysis/AllCells/cc_none/curr_seurat_object.rds"))
curr_seurat_object <- read_rds(file = glue("{getwd()}/code_out/{prefix}{Round}_cluster/{prefix}seurat_object.rds")) 

### Remove the RNA assay to make it smaller
#curr_seurat_object[['RNA']] <- NULL

### Subset the FULL dataset to only the NoCre cells, then remove the full dataset to save on memory
#Idents(curr_seurat_object) <- "Sample"
Idents(curr_seurat_object) <- "genotype"
WT_curr_seurat_object <- subset(curr_seurat_object, idents = ident.2)

table(Idents(curr_seurat_object))
table(Idents(WT_curr_seurat_object))

DimPlot(WT_curr_seurat_object, label=TRUE)

if(Round == "Round0"){ Idents(WT_curr_seurat_object) <- "Round0Cluster" }
if(Round == "Round1"){ Idents(WT_curr_seurat_object) <- "Round1Cluster" }
if(Round == "Round2"){ Idents(WT_curr_seurat_object) <- "Round2Cluster" }

DimPlot(WT_curr_seurat_object, label=TRUE)

#rm(curr_seurat_object)

### FindAllMarkers() of one vs the other clusters (using only NoCre cells) to get genes that distinguish clusters at this low resolution
### Since we want large lists for GSEA, rather than fewer clear marker genes, we use very permissive filters compared to previous FindMarkers() calls. 
  ### !!! This will take awhile, so we create WT_curr.markers_FAM, but use WT_curr.markers afterward so that if we mess something up, we can refer back to WT_curr.markers_FAM.
  WT_curr.markers_FAM <- FindAllMarkers(WT_curr_seurat_object, only.pos = FALSE, min.pct = min.pct, logfc.threshold = logfc.threshold, return.thresh = 1)

  WT_curr.markers <- WT_curr.markers_FAM %>% mutate(signed_negLog10_p_val_adj = -log10(p_val_adj) * sign(avg_log2FC))
  WT_curr.markers <- WT_curr.markers %>% mutate(signed_negLog10_p_val = -log10(p_val) * sign(avg_log2FC))
  ### We arrange by signed_negLog10_p_val because MANY of the genes have adjusted values equal to 1.
  WT_curr.markers <- WT_curr.markers %>% arrange(desc(signed_negLog10_p_val)) %>% arrange(cluster)

  max_signed_padj <- max(abs(WT_curr.markers$signed_negLog10_p_val_adj [is.finite(WT_curr.markers$signed_negLog10_p_val_adj)]))

  WT_curr.markers <- WT_curr.markers %>% mutate(signed_negLog10_p_val_adj = ifelse(is.finite(signed_negLog10_p_val_adj), signed_negLog10_p_val_adj, max_signed_padj * sign(avg_log2FC)))

  write.table(x=WT_curr.markers, file = glue("{Round}Clusters","_EachClustervAllOthers","_NoCreOnly","_minpct",min.pct,"_log2fc",logfc.threshold,"_230101.tsv"), sep = "\t")

print(Sys.time()); print(""); print("Line ~980")  
print(glue("FindAllMarkers() output written to: {Round}Clusters","_EachClustervAllOthers","_NoCreOnly","_minpct",min.pct,"_log2fc",logfc.threshold,"_230101.tsv"))

}


## ---- Create .rnk files for INTERcluster GSEA using only NoCre cells----------
### !!! Add outdir for this block

WT_curr.markers <- read.table(file = glue("{Round}Clusters","_EachClustervAllOthers","_NoCreOnly","_minpct",min.pct,"_log2fc",logfc.threshold,"_230101.tsv"), sep = "\t")


if(RUNALLSTEPS == FALSE){ print("Skipping this step.") };  
if(RUNALLSTEPS == TRUE){


### ___ This should maybe go up a block
### Create .rnk files for GSEA input for each cluster (note: the values of WT_curr.markers$cluster correspond to the values of curr_seurat_object$Round#_Cluster)
  for(i in levels(WT_curr.markers$cluster)){
    print(i)
    temp <- mutate(.data = filter(WT_curr.markers, cluster == i ) %>% dplyr::select( gene, signed_negLog10_p_val_adj ) )
    write.table(x = temp, file = glue("GSEA_input/{Round}Clusters_NoCreCluster",i,"vAllOthers_230101.rnk"), quote = FALSE, sep = "\t", row.names = FALSE, col.names = FALSE)
  }

print(Sys.time()); print(""); print("Line ~993")  
print(glue("{length(WT_curr.markers$cluster)} .rnk files written to GSEA_input/"))



### Repeat for just the round1 clusters in the GMC compared only to each other. This uses "clusters_to_use" defined at the start of this file to subset out the clusters of interest.
if(Round == "Round0"){ Idents(WT_curr_seurat_object) <- "Round0Cluster" }
if(Round == "Round1"){ Idents(WT_curr_seurat_object) <- "Round1Cluster" }
if(Round == "Round2"){ Idents(WT_curr_seurat_object) <- "Round2Cluster" }

WT_curr_seurat_object <- subset(WT_curr_seurat_object, idents = clusters_to_use)

DimPlot(WT_curr_seurat_object, label=TRUE)

### FindAllMarkers() of one vs the other myeloid clusters (using only NoCre cells) to get genes that distinguish clusters at this high resolution 
### Since we want large lists for GSEA, rather than fewer clear marker genes, we use very permissive filters compared to previous FindMarkers() calls. 

### !!! This will take awhile, so we create WT_curr.markers_FAM, but use WT_curr.markers afterward so that if we mess something up, we can refer back to WT_curr.markers_FAM.
WT_curr.markers_FAM <- FindAllMarkers(WT_curr_seurat_object, only.pos = FALSE, min.pct = min.pct, logfc.threshold = logfc.threshold, return.thresh = 1)

WT_curr.markers <- WT_curr.markers_FAM %>% mutate(signed_negLog10_p_val_adj = -log10(p_val_adj) * sign(avg_log2FC))
WT_curr.markers <- WT_curr.markers %>% mutate(signed_negLog10_p_val = -log10(p_val) * sign(avg_log2FC))
### We arrange by signed_negLog10_p_val because MANY of the genes have adjusted values equal to 1.
WT_curr.markers <- WT_curr.markers %>% arrange(desc(signed_negLog10_p_val)) %>% arrange(cluster)

max_signed_padj <- max(abs(WT_curr.markers$signed_negLog10_p_val_adj))

WT_curr.markers <- WT_curr.markers %>% mutate(signed_negLog10_p_val_adj = ifelse(is.finite(signed_negLog10_p_val_adj), signed_negLog10_p_val_adj, max_signed_padj * sign(avg_log2FC)))

write.table(x=WT_curr.markers, file = glue("{Round}Clusters","_EachMacroClustervOtherMacros","_NoCreOnly","_minpct",min.pct,"_log2fc",logfc.threshold,"_230101.tsv"), sep = "\t")


### Create .rnk files for GSEA input for each cluster (note: the values of WT_curr.markers$cluster correspond to the values of curr_seurat_object$Round#_Cluster)
for(i in levels(WT_curr.markers$cluster)){
  print(i)
  temp <- mutate(.data = filter(WT_curr.markers, cluster == i ) %>% dplyr::select( gene, signed_negLog10_p_val_adj ) )
  write.table(x = temp, file = glue("GSEA_input/{Round}Clusters_NoCreCluster",i,"vOtherMacros_230101.rnk"), quote = FALSE, sep = "\t", row.names = FALSE, col.names = FALSE)
}



}


## -----------------------------------------------------------------------------

#geneList <-           read.table(file = glue("GSEA_input/{Round}Clusters_NoCreCluster",9,"vAllOthers_230101.rnk"), col.names = 1)

#geneList <-           read_delim(file = glue("GSEA_input/{Round}Clusters_NoCreCluster",9,"vAllOthers_230101.rnk"))


geneList <-           read.table(file = glue("GSEA_input/{Round}Clusters_NoCreCluster",9,"vAllOthers_230101.rnk"))[,2]
names(geneList) <- read.table(file = glue("GSEA_input/{Round}Clusters_NoCreCluster",9,"vAllOthers_230101.rnk"))[,1]
#colnames(geneList) <- c("Signed-log10pValAdj")

#range(geneList[,2])

#enrich <- GSEA(geneList = geneList, TERM2GENE = Mh, pvalueCutoff = 1, pAdjustMethod = "BH")

enrich <- GSEA(geneList, 
             minGSSize = 15,
             maxGSSize = 500,
             TERM2GENE = Mh,
             pvalueCutoff = 0.1,
             pAdjustMethod = "BH")




## ---- Run GSEA GUI and combine summaries across clusters into up and down files before manually adding cluster number----
if(FALSE){
### Download .rnk files to run GSEA GUI locally (if running from Agate)

### Locally:
# rsync -avzP bmcc@agate.msi.umn.edu:"/home/schwertf/shared/braedan_AE_scrnaseq_220502/C2/GSEA_input/*.rnk" /Users/minnow/Documents/Proj/schwertf/Schwertf_Project_028_AE_scRNAseq_22May/seurat_schwertfProj028/C2/GSEA_input/


### Run GSEA using the gene lists specified by Alexis as per an email sent Sept 15, 2022:
# "Hi Braedan, 
#Thanks for the update. We'd like to use these lists: 
#MH 
#M2 (Biocarta, reatome, wikipathways)
#M5 (GO and MPT)
#Thanks,
#Alexis"

### Since the Sept 2022 update to MSigDB, there are not official Mouse gene set lists. This makes comparisons easier than previous iterations that had to remap mouse genes human genes. Here is an example run.

#gsea-cli.sh GSEAPreranked -gmx ftp.broadinstitute.org://pub/gsea/msigdb/mouse/gene_sets/mh.all.v2022.1.Mm.symbols.gmt,ftp.broadinstitute.org://pub/gsea/msigdb/mouse/gene_sets/m2.all.v2022.1.Mm.symbols.gmt,ftp.broadinstitute.org://pub/gsea/msigdb/mouse/gene_sets/m5.all.v2022.1.Mm.symbols.gmt -collapse No_Collapse -mode Abs_max_of_probes -norm meandiv -nperm 1000 -rnd_seed 149 -rnk /Users/minnow/Documents/Proj/schwertf/Schwertf_Project_028_AE_scRNAseq_22May/seurat_schwertfProj028/C2/GSEA_input/Round1Clusters_NoCreCluster1vOtherMacros_220819.rnk -scoring_scheme classic -rpt_label GSEA_4.3.0_Round1Clusters_NoCreCluster1vOtherMacros_MHM2M5_220819 -create_svgs true -include_only_symbols true -make_sets true -plot_top_x 20 -set_max 500 -set_min 15 -zip_report true -out /Users/minnow/Documents/Proj/schwertf/Schwertf_Project_028_AE_scRNAseq_22May/seurat_schwertfProj028/C2/GSEA_output

# In total, we are looking at 13,321 gene sets: 50 hallmark gene sets in MH; 932 chemical and genetic perturbations and 1687 canonical pathways in M2; and 10560 gene ontology and 92 mammalian tumor phenotype lists in M5. Over half the gene sets (7775) are GO:Biological Process gene sets. With our min and max gene set sizes, we end up with ~5700 gene lists per sample.

# When all are run, combine the results summaries into a single file for Up and Down lists
### There should be several files named with e.g. "GSEA_4.3.0_Round1Clusters_NoCreCluster1vOtherMacros_MHM2M5_220819" in the name
system("pwd")
system(glue("ls -1 GSEA_output/GSEA_4.3.0_{Round}Clusters_NoCreCluster*vOtherMacros_MHM2M5_220819*/gsea_report_for_na_neg_*tsv"))
system("cat GSEA_output/GSEA_4.3.0_Round1Clusters_NoCreCluster*vOtherMacros_MHM2M5_220819*/gsea_report_for_na_pos_*tsv > GSEA_output/GSEA_4.3.0_Round1Clusters_EachNoCreClustervOtherMacroClusters_MHM2M5_220819_UP.tsv")
system("cat GSEA_output/GSEA_4.3.0_Round1Clusters_NoCreCluster*vOtherMacros_MHM2M5_220819*/gsea_report_for_na_neg_*tsv > GSEA_output/GSEA_4.3.0_Round1Clusters_EachNoCreClustervOtherMacroClusters_MHM2M5_220819_DOWN.tsv")


system("cat GSEA_output/GSEA_4.3.0_Round1Clusters_EachNoCreClustervOtherMacroClusters_MHM2M5_220819_UP.tsv GSEA_output/GSEA_4.3.0_Round1Clusters_EachNoCreClustervOtherMacroClusters_MHM2M5_220819_DOWN.tsv > GSEA_output/GSEA_4.3.0_Round1Clusters_EachNoCreClustervOtherMacroClusters_MHM2M5_220819.tsv")

# BMM Note: I added two extra columns named "Clustering Round" and "Cluster" to denote the round of clustering and the cluster the gene list was up or down in and sent it as an excel file for easy viewing. There are ~9,000 rows in each file, so I expect we may be focusing in on candidate gene lists a bit further in the future.

### Upload .tsv file to Agate again if running from there.
# rsync -avzP GSEA_output/*tsv bmcc@agate.msi.umn.edu:"/home/schwertf/shared/braedan_AE_scrnaseq_220502/C2/GSEA_output/" 

temp <- read_tsv(file="GSEA_output/GSEA_4.3.0_Round1Clusters_EachNoCreClustervOtherMacroClusters_MHM2M5_220819.tsv")


}


## ---- Intracluster NoCreVLyveCre----------------------------------------------
print(Sys.time()); print("");
print(glue("Starting INTRAcluster GSEA"));

if(RUNALLSTEPS == FALSE){ print("Skipping this step.") };  
if(RUNALLSTEPS == TRUE){



#if(RUNALLSTEPS == TRUE){
###Find Markers and Create .rnk files for GSEA of NoCre vs LyveCre cells for each cluster
### BMM edit 220830: Now using signed_negLog10_p_val instead of signed_negLog10_p_val_adj since the latter gave so many 0s and thus no way to order many genes

#curr_seurat_object <- read_rds(file = glue(getwd(),"/code_out/101_Round1_cluster_analysis/AllCells/cc_none/curr_seurat_object.rds"))
curr_seurat_object <- read_rds(file = glue("{getwd()}/code_out/{prefix}{Round}_cluster/{prefix}seurat_object.rds")) 

#Idents(curr_seurat_object) <- "Round1_Cluster"

if(Round == "Round0"){ Idents(curr_seurat_object) <- "Round0Cluster" }
if(Round == "Round1"){ Idents(curr_seurat_object) <- "Round1Cluster" }
if(Round == "Round2"){ Idents(curr_seurat_object) <- "Round2Cluster" }

DimPlot(WT_curr_seurat_object, label=TRUE)


### !!! ___ This may throw errors if there are too few cells in a cluster x genotype combination!
### FindAllMarkers() of NoCre vs LyveCre cells for each {Round}_Cluster to get genes that distinguish the samples in each {Round}_Cluster.
### Since we want large lists for GSEA, rather than fewer clear marker genes, we use very permissive filters compared to previous FindMarkers() calls. 

for(i in levels(Idents(curr_seurat_object))){
  #for(i in SubClusters){
    print(i)
  ### Create temp_curr_seurat_object with only the subset of cells in one Cluster.
  temp_curr_seurat_object <- subset(curr_seurat_object, idents = i)
  Idents(temp_curr_seurat_object) <- "genotype"
  #DimPlot(temp_curr_seurat_object, label=TRUE)
  
  # Since we only have two values when Idents() is set to genotype: "NoCre" and "LyveCre", we run FindAllMarkers(), then add the cluster number (stored in "i") later.
  # We use ident.1 as our reference. This should be the treatment group in most cases. 
  print(ident.1)
  temp_gmc.markers_FAM <- FindAllMarkers(temp_curr_seurat_object, only.pos = FALSE, min.pct = min.pct, logfc.threshold = logfc.threshold, return.thresh = 1)
  
  temp_gmc.markers <- temp_gmc.markers_FAM %>% filter(cluster == ident.1) %>% mutate(signed_negLog10_p_val_adj = -log10(p_val_adj) * sign(avg_log2FC))
  temp_gmc.markers <- temp_gmc.markers %>% mutate(signed_negLog10_p_val = -log10(p_val) * sign(avg_log2FC))
  ### We arrange by signed_negLog10_p_val because MANY of the genes have adjusted values equal to 1.
  temp_gmc.markers <- temp_gmc.markers %>% arrange(desc(signed_negLog10_p_val)) %>% arrange(cluster)
  
  max_signed_padj <- max(abs(temp_gmc.markers$signed_negLog10_p_val_adj))
  
  temp_gmc.markers <- temp_gmc.markers %>% mutate(signed_negLog10_p_val_adj = ifelse(is.finite(signed_negLog10_p_val_adj), signed_negLog10_p_val_adj, max_signed_padj * sign(avg_log2FC)))
  
  ### Overwrite the "Sample" value sorted in the "cluster" column with the cluster number set to "i"
  temp_gmc.markers <- temp_gmc.markers %>% mutate(cluster = i)
  ### Write a .tsv for the Round1Cluster results
  write.table(x=temp_gmc.markers, file = glue("GSEA_input/{Round}Clusters","_Cluster",i,"_",ident.1,"V",ident.2,"_minpct",min.pct,"_log2fc",logfc.threshold,"_230101.tsv"), sep = "\t")
  
  ### Create .rnk files for GSEA input for each cluster (note: the values of WT_gmc.markers$cluster correspond to the values of curr_seurat_object$Round1_Cluster)
  temp <- dplyr::select(.data = temp_gmc.markers, gene, signed_negLog10_p_val_adj )
  write.table(x = temp, file = glue("GSEA_input/{Round}Clusters","_Cluster",i,"_",ident.1,"V",ident.2,"_minpct",min.pct,"_log2fc",logfc.threshold,"_230101.rnk"), quote = FALSE, sep = "\t", row.names = FALSE, col.names = FALSE)

}


### !!! ___ Since we are doing a bunch of different comparisons, we need to paste the output together for all the clusters. This is not needed in the previous iteration since we have one less level of complexity bc we throw out the LyveCre cells and can use FindAllMarkers() across all the comparisons we are interested in.
filesToCat <- list.files(path="GSEA_input/", pattern=glue("{Round}Clusters_Cluster.*tsv"))
allTumClusters <- data.frame()
for(i in 1:length(filesToCat)){
  if(i == 1){allTumClusters <- read.table(file=glue("GSEA_input/",filesToCat[i]), row.names = NULL) }
    else{allTumClusters <- rbind(allTumClusters, read.table(file=glue("GSEA_input/",filesToCat[i]), row.names = NULL)) }
}
dim(allTumClusters)

### Write a .tsv for the Cluster results
write.table(x=allTumClusters, file = glue("{Round}Clusters","_EachCluster","_",ident.1,"V",ident.2,"_minpct",min.pct,"_log2fc",logfc.threshold,"_220819.tsv"), sep = "\t")



}


## ---- Run GSEA GUI and combine summaries across clusters into up and down files before adding cluster number manually----
if(FALSE){

### Run GSEA using the gene lists specified by Alexis as per an email sent Sept 15, 2022:
# "Hi Braedan, 
#Thanks for the update. We'd like to use these lists: 
#MH 
#M2 (Biocarta, reatome, wikipathways)
#M5 (GO and MPT)
#Thanks,
#Alexis"

### Since the Sept 2022 update to MSigDB, there are now official Mouse gene set lists. This makes comparisons easier than previous iterations that had to remap mouse genes human genes. Here is an example run.

### Previous runs used remapping to humans and/or GSEA within R (see example from Yinjie in next block). Ideally, future runs will be able to use the new Mouse gene sets with command line GSEA.

#gsea-cli.sh GSEAPreranked -gmx ftp.broadinstitute.org://pub/gsea/msigdb/mouse/gene_sets/mh.all.v2022.1.Mm.symbols.gmt,ftp.broadinstitute.org://pub/gsea/msigdb/mouse/gene_sets/m2.all.v2022.1.Mm.symbols.gmt,ftp.broadinstitute.org://pub/gsea/msigdb/mouse/gene_sets/m5.all.v2022.1.Mm.symbols.gmt -collapse No_Collapse -mode Abs_max_of_probes -norm meandiv -nperm 1000 -rnd_seed 149 -rnk /Users/minnow/Documents/Proj/schwertf/Schwertf_Project_028_AE_scRNAseq_22May/seurat_schwertfProj028/C2/GSEA_input/Round1Clusters_NoCreCluster0vAllOthers_220819.rnk -scoring_scheme classic -rpt_label GSEA_4.3.0_Round1Clusters_NoCreCluster0vOtherMacros_MHM2M5_220819 -create_svgs true -include_only_symbols true -make_sets true -plot_top_x 20 -set_max 500 -set_min 15 -zip_report true -out /Users/minnow/Documents/Proj/schwertf/Schwertf_Project_028_AE_scRNAseq_22May/seurat_schwertfProj028/C2/GSEA_output


# In total, we are looking at 13,321 gene sets: 50 hallmark gene sets in MH; 932 chemical and genetic perturbations and 1687 canonical pathways in M2; and 10560 gene ontology and 92 mammalian tumor phenotype lists in M5. Over half the gene sets (7775) are GO:Biological Process gene sets. With our min and max gene set sizes, we end up with ~5700 gene lists per sample.

# When all are run, combine the results summaries into a single file for Up and Down lists
### There should be several files named with e.g. "GSEA_4.3.0_Round1Clusters_NoCreCluster1vOtherMacros_MHM2M5_220819" in the name
system("pwd") 
system("ls -1 GSEA_output/GSEA_4.3.0_Round1Clusters_Cluster*_LyveCreVNoCre_MHM2M5_220819*/gsea_report_for_na_neg_*tsv")
system("cat GSEA_output/GSEA_4.3.0_Round1Clusters_Cluster*_LyveCreVNoCre_MHM2M5_220819*/gsea_report_for_na_pos_*tsv > GSEA_output/GSEA_4.3.0_Round1Clusters_EachCluster_LyveCreVNoCre_MHM2M5_220819_UP.tsv")
system("cat GSEA_output/GSEA_4.3.0_Round1Clusters_Cluster*_LyveCreVNoCre_MHM2M5_220819*/gsea_report_for_na_neg_*tsv > GSEA_output/GSEA_4.3.0_Round1Clusters_EachCluster_LyveCreVNoCre_MHM2M5_220819_DOWN.tsv")

system("cat GSEA_output/GSEA_4.3.0_Round1Clusters_EachCluster_LyveCreVNoCre_MHM2M5_220819_UP.tsv GSEA_output/GSEA_4.3.0_Round1Clusters_EachCluster_LyveCreVNoCre_MHM2M5_220819_DOWN.tsv > GSEA_output/GSEA_4.3.0_Round1Clusters_EachCluster_LyveCreVNoCre_MHM2M5_220819.tsv")

# BMM Note: I added two extra columns named "Clustering Round" and "Cluster" to denote the round of clustering and the cluster the gene list was up or down in and sent it as an excel file for easy viewing. There are ~9,000 rows in each file, so I expect we may be focusing in on candidate gene lists a bit further in the future.

### Upload .tsv file to Agate again if running from there.
# rsync -avzP GSEA_output/*tsv bmcc@agate.msi.umn.edu:"/home/schwertf/shared/braedan_AE_scrnaseq_220502/C2/GSEA_output/" 

temp <- read_tsv(file="GSEA_output/GSEA_4.3.0_Round1Clusters_EachCluster_LyveCreVNoCre_MHM2M5_220819.tsv")

}

