#!/usr/bin/env Rscript

### Bash requirements (via module load or specified in OOD start up box)
# module load gcc/8.1.0
# module load udunits/2.2.26_gcc8.1.0
# module load geos/3.7.1
# module unload R
# module load 
# module load R/4.2.2-openblas
# module load hdf5
# cd /home/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/
  

### Do some checks that the setup is correct:
# Should return 4.2.2 and some extra info
version 

system("which R")

### Make sure it is looking in the correct location for libraries
.libPaths()

### Load required packages
### If needed, install packages to my library, since the shared one at /panfs/roc/msisoft/R/4.2.2-openblas/lib64/R/library is not writable.
# install.packages('Seurat', lib='/panfs/jay/groups/3/lmnp/bmcc/R/x86_64-pc-linux-gnu-library/4.2')

# BiocManager::install("biomaRt", lib='/panfs/jay/groups/3/lmnp/bmcc/R/x86_64-pc-linux-gnu-library/4.2')


library(Seurat) #cran
library(glue) 
library(dplyr)
library(purrr)
library(readr)
library(stringr)


#######################################################################
# Script parameters
#######################################################################
# Clear out any objects from workspace
rm(list = ls(all.names = TRUE))

# Set R working directory and project name here.
proj_dir <- "/home/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502"
setwd(proj_dir)

project_name <- "C1_NoCreVlyveCre"

# Set up output directory and filename prefix
out_dir <- glue("{proj_dir}/code_out/031_filter_by_rna")
if (!dir.exists(glue("{out_dir}"))) {dir.create(glue("{out_dir}"), recursive = TRUE)}
setwd(glue("{out_dir}"))
prefix <- "031_"
if (!dir.exists(out_dir)) {dir.create(out_dir, recursive = TRUE)}


#######################################################################
# Analysis
#######################################################################


# ---------------------------------------------------------------------
# Read input data
# ---------------------------------------------------------------------

### !!! If using previous code block, use this: samples <- readRDS(glue("{proj_dir}/code_out/021_samples/021_samples.rds"))
### 230317 BMM edit for two experiments:
samples <- read_tsv(glue("{proj_dir}/input/metadata_C1.tsv")) %>% purrr::modify_at(vars(everything()), factor)

saveRDS(samples, file = glue("{out_dir}/{prefix}samples.rds"))

### Working from Todd's workflows, namely this one for this script: https://github.umn.edu/knut0297org/farrarm_todd_citeseq_nilotinib_pdl1_20210109/blob/663ae278f1b2aa0b0b36c762f61a31fec193e272/code/031_filter_by_rna.R

### Since we have two separate toaster runs, I have to combine the CellRanger output stored in filtered_feature_bc_matrix/
### I am processing them separately, then using merge() later.
matrix_dir <- "/home/schwertf/shared/disaster_recovery/bmcc/braedan_AE_scrnaseq_220502/code_out/cellranger_C1_220510/Schwertf_Project_028/outs/filtered_feature_bc_matrix"

features <- read_tsv(glue("{matrix_dir}/features.tsv.gz"), col_names = FALSE) %>%
  dplyr::rename(ensembl_id = "X1", symbol = "X2", data_type = "X3")

any(duplicated(features[, 1]))
# FALSE

data <- Read10X(data.dir=matrix_dir, gene.column = 2)

# Verify that cell barcodes were all detected by both RNA and HTO. 
all.equal(colnames(data[["Gene Expression"]]), colnames(data[["Antibody Capture"]]))
# TRUE

saveRDS(data, file = glue("{out_dir}/{prefix}data.rds"))



# ---------------------------------------------------------------------
# Gene expression and HTO datasets
# ---------------------------------------------------------------------

rna_counts1 <- data[["Gene Expression"]]
hto_counts1 <- data[["Antibody Capture"]]

rna_features1 <- features %>%
  dplyr::filter(data_type == "Gene Expression")
hto_features1 <- features %>%
  dplyr::filter(data_type == "Antibody Capture") %>%
  dplyr::rename(biolegend_name = "ensembl_id", sample_name = "symbol")


# ---------------------------------------------------------------------
# Create Seurat object
# ---------------------------------------------------------------------


# Create Seurat object with RNA counts
curr_seurat_object <- CreateSeuratObject(counts = rna_counts1, assay = "RNA", project = project_name, min.cells = 0, min.features = 0)

### Adding below to see if that fixes an issue.
# Add the HTO dataset
curr_seurat_object[["HTO"]] <- CreateAssayObject(counts = hto_counts1, assay = "HTO")

# Add RNA and HTO features metadata to seurat object
curr_seurat_object@assays$RNA@meta.features$ensembl_id <- rna_features1$ensembl_id
curr_seurat_object@assays$RNA@meta.features$symbol <- rna_features1$symbol
curr_seurat_object@assays$RNA@meta.features$data_type <- rna_features1$data_type
curr_seurat_object@assays$HTO@meta.features$biolegend_name <- hto_features1$biolegend_name
curr_seurat_object@assays$HTO@meta.features$sample_name <- hto_features1$sample_name
curr_seurat_object@assays$HTO@meta.features$data_type <- hto_features1$data_type


# ---------------------------------------------------------------------
# split cells into samples using HTO tags
# ---------------------------------------------------------------------

# Normalize HTO data, here we use "LogNormalize" instead of centered log-ratio (CLR) transformation
curr_seurat_object <- NormalizeData(curr_seurat_object, assay = "HTO", normalization.method = "CLR")

# Use the Seurat function HTODemux() to assign single cells back to their sample origins.
### ___ !!! The default for "positive.quantile" is 0.99. We are using a lower value to avoid doublets.
curr_seurat_object <- HTODemux(curr_seurat_object, assay = "HTO", positive.quantile = 0.8, verbose = TRUE)
table(curr_seurat_object@meta.data$HTO_classification)

# Group cells based on the max HTO signal
#Idents(curr_seurat_object) <- "HTO_maxID"
Idents(curr_seurat_object) <- "HTO_classification"
RidgePlot(curr_seurat_object, assay = "HTO", features = rownames(curr_seurat_object[["HTO"]])[1:2], ncol = 2)
#RidgePlot(curr_seurat_object, assay = "HTO", features = rownames(curr_seurat_object[["HTO"]]))

FeatureScatter(curr_seurat_object, feature1 = "hto_tum1NoCre", feature2 = "hto_tum1LyveCre", group.by = "HTO_classification", pt.size = 0.2)
#FeatureScatter(curr_seurat_object, feature1 = "hto_tum1NoCre", feature2 = "nCount_RNA", group.by = "HTO_classification", pt.size = 0.2)

  #hist(curr_seurat_object@meta.data$nCount_RNA, xlim=c(0,10000), breaks = 2000)

### Also record the proportion of HTO reads that are from the sample with the most reads and add a meta.data flag for those that are 90%+ from one sample
HTO_maxProportion <- apply(curr_seurat_object@assays$HTO@counts, 2, max) / curr_seurat_object@meta.data$nCount_HTO
curr_seurat_object <- AddMetaData(object = curr_seurat_object, metadata = HTO_maxProportion, col.name = 'HTO_maxProportion')

HTO_altProportion <- 1 - curr_seurat_object@meta.data$HTO_maxProportion
curr_seurat_object <- AddMetaData(object = curr_seurat_object, metadata = HTO_altProportion, col.name = 'HTO_altProportion')

HTO_90plus <- curr_seurat_object@meta.data$HTO_maxProportion >= 0.90
curr_seurat_object <- AddMetaData(object = curr_seurat_object, metadata = HTO_90plus, col.name = 'HTO_90plus')

FeatureScatter(curr_seurat_object, feature1 = "hto_tum1NoCre", feature2 = "hto_tum1LyveCre", group.by = "HTO_classification", pt.size = 0.2)
FeatureScatter(curr_seurat_object, feature1 = "hto_tum1NoCre", feature2 = "hto_tum1LyveCre", group.by = "HTO_90plus", pt.size = 0.2)


### Look at the distributions of potentially problematic meta.data fields
hist(curr_seurat_object@meta.data$HTO_maxProportion, breaks = 100)
hist(curr_seurat_object@meta.data$nCount_RNA, breaks = 10000, xlim = c(0,5000))
  
# ---------------------------------------------------------------------
# Standard preprocessing
# ---------------------------------------------------------------------

DefaultAssay(curr_seurat_object) <- "RNA"

# Identify the mitochondrial genes
str_subset(rownames(curr_seurat_object), "^mt-")


# Store mitochondrial percentage in object meta data
curr_seurat_object <- PercentageFeatureSet(curr_seurat_object, pattern = "^mt-", col.name = "percent_mito")
hist(curr_seurat_object@meta.data$percent_mito, breaks = 100)



pdf(glue("{out_dir}/{prefix}qc_metrics_violin.pdf"), width = 10, height = 6)
VlnPlot(curr_seurat_object, features = c("nFeature_RNA", "nCount_RNA", "percent_mito"), ncol = 3)
dev.off()


m <- curr_seurat_object@meta.data$percent_mito
m1 <- quantile(as.numeric(m), c(0.99))
pdf(glue("{out_dir}/{prefix}qc_percent_mito.pdf"), width = 10, height = 6)
plot(m, main = "percent_mito, 99.5% thresholds", ylab = "percent_mito", xlab = "cell_id")
abline(h = m1[1])
abline(h = m1[2])
dev.off()



plot1 <- FeatureScatter(curr_seurat_object, feature1 = "nCount_RNA", feature2 = "percent_mito")
plot2 <- FeatureScatter(curr_seurat_object, feature1 = "nCount_RNA", feature2 = "nFeature_RNA")
pdf(glue("{out_dir}/{prefix}qc_metrics_scatter.pdf"), width = 8, height = 6)
# FYI: Plots are combined via "patchwork" R package methods
plot1 + plot2
dev.off()


f <- curr_seurat_object@meta.data$nFeature_RNA
f1 <- quantile(as.numeric(f), c(0.01, 0.99))
pdf(glue("{out_dir}/{prefix}qc_nFeature_RNA.pdf"), width = 8, height = 6)
plot(f, main = "nFeature_RNA, 0.5% and 99.5% thresholds", ylab = "nFeature_RNA", xlab = "cell_id")
abline(h = f1[1])
abline(h = f1[2])
dev.off()


c <- curr_seurat_object@meta.data$nCount_RNA
c1 <- quantile(as.numeric(c), c(0.01, 0.99))
pdf(glue("{out_dir}/{prefix}qc_nCount_RNA.pdf"), width = 8, height = 6)
plot(c, main = "nCount_RNA, 2% and 98% thresholds", ylab = "nCount_RNA", xlab = "cell_id")
abline(h = c1[1])
abline(h = c1[2])
dev.off()









# Using this vignette as a guide: https://satijalab.org/seurat/articles/hashing_vignette.html

# joint.bcs <- intersect(colnames(pbmc.umis), colnames(pbmc.htos))

# Normalize RNA data with log normalization
curr_seurat_object <- NormalizeData(curr_seurat_object)

# Find and scale variable features
curr_seurat_object <- FindVariableFeatures(curr_seurat_object, selection.method = "mean.var.plot")
curr_seurat_object <- ScaleData(curr_seurat_object, features = VariableFeatures(curr_seurat_object))

### Added previously, so skipping
# Add HTO data as a new assay independent from RNA
# curr_seurat_object[["HTO"]] <- CreateAssayObject(counts = curr_seurat_object@assays$HTO@counts)


#curr_seurat_object[["HTO"]] <- CreateAssayObject(counts = hto_counts1)

# Add RNA and HTO features metadata to seurat object
#curr_seurat_object@assays$HTO@meta.features$biolegend_name <- hto_features1$biolegend_name
#curr_seurat_object@assays$HTO@meta.features$sample_name <- hto_features1$sample_name
#curr_seurat_object@assays$HTO@meta.features$data_type <- hto_features1$data_type




#######################################################################
# Save session info
#######################################################################

# ---------------------------------------------------------------------
# Save the Seurat Object
# ---------------------------------------------------------------------
write_rds(curr_seurat_object, file = glue("{out_dir}/{prefix}seurat_object.rds"))



# ---------------------------------------------------------------------
# Save all R objects
# ---------------------------------------------------------------------

# save.image(file = glue("{out_dir}/{prefix}r_objects.RData"))
write_tsv(data.frame(RData_objects = ls()), glue("{out_dir}/{prefix}r_objects.txt"))

